﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Optimization.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Core;
using HeuristicLab.Problems.BinPacking3D;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.BinPacking.Views {
    [View("Permutation 3D Bin Packing Problem View")]
    [Content(typeof(BinPacking3D.PermutationProblem), IsDefaultView = true)]
    public partial class BinPacking3DProblemView : ProblemView {

      public new BinPacking3D.PermutationProblem Content {
        get { return (BinPacking3D.PermutationProblem)base.Content; }
        set { base.Content = value; }
      }

      public BinPacking3DProblemView() {
          InitializeComponent();
      }

      protected override void OnContentChanged() {
        base.OnContentChanged();
        sortingComboBox.SelectedIndex = 0;
        heuristicComboBox.SelectedIndex = 0;
      }

      private void heuristicComboBox_SelectedIndexChanged(object sender, EventArgs e) {
        if (Content == null) return;
        if (heuristicComboBox.SelectedItem.ToString().CompareTo("Best Fit (Free Volume)") == 0) {
          
        }
      }

      private void sortingComboBox_SelectedIndexChanged(object sender, EventArgs e) {
        if (Content == null) return;
        deltaLabel.Visible = false;
        deltaTextBox.Visible = false;
        var items = new ItemList<PackingItem>(Content.Items);

         if (sortingComboBox.SelectedItem.ToString().CompareTo("Volume-Height") == 0) {
           var sortedItems = new ItemList<PackingItem>(items.OrderByDescending(x => x.Depth * x.Width * x.Height).ThenByDescending(x => x.Height));
           Content.Items = sortedItems.AsReadOnly();
          }
          else if (sortingComboBox.SelectedItem.ToString().CompareTo("Height-Volume") == 0) {
            var sortedItems = new ItemList<PackingItem>(items.OrderByDescending(x => x.Height).ThenByDescending(x => x.Depth * x.Width * x.Height));
            Content.Items = sortedItems.AsReadOnly();
          }
          else if (sortingComboBox.SelectedItem.ToString().CompareTo("Area-Height") == 0) {
            var sortedItems = new ItemList<PackingItem>(items.OrderByDescending(x => x.Depth * x.Width).ThenByDescending(x => x.Height));
            Content.Items = sortedItems.AsReadOnly();
          }
          else if (sortingComboBox.SelectedItem.ToString().CompareTo("Height-Area") == 0) {
            var sortedItems = new ItemList<PackingItem>(items.OrderByDescending(x => x.Height).ThenByDescending(x => x.Depth * x.Width));
            Content.Items = sortedItems.AsReadOnly();
          }
          else if (sortingComboBox.SelectedItem.ToString().CompareTo("Clustered Area-Height") == 0) {
            deltaLabel.Visible = true;
            deltaTextBox.Visible = true;
            deltaTextBox.Clear();
          }
          else if (sortingComboBox.SelectedItem.ToString().CompareTo("Clustered Height-Area") == 0) {
            deltaLabel.Visible = true;
            deltaTextBox.Visible = true;
            deltaTextBox.Clear();
          }
        }

        private void deltaTextBox_TextChanged(object sender, EventArgs e) {
          Dictionary<int, ItemList<PackingItem>> clusters = new Dictionary<int, ItemList<PackingItem>>();
          var items = new ItemList<PackingItem>(Content.Items);

          if (deltaTextBox.Text.Length > 0) {
           int delta = Convert.ToInt32(deltaTextBox.Text);
           
           if (sortingComboBox.SelectedItem.ToString().CompareTo("Clustered Area-Height") == 0) {
             // Cluster packing items with respect to area = width * depth
             for (int i = 0; i < items.Count; i++) {
               double clusterRange = items[i].TargetBin.Width * items[i].TargetBin.Depth * delta / 100.0;
               int clusterId = (int)(Math.Ceiling(items[i].Width * items[i].Depth / clusterRange));

               if (clusters.ContainsKey(clusterId)) clusters[clusterId].Add(items[i]);
               else clusters.Add(clusterId, new ItemList<PackingItem> {items[i]});
              }           

              // Sort packing items inside the cluster where they belong with respect to height 
              foreach (KeyValuePair<int, ItemList<PackingItem>> entry in clusters) {
                clusters[entry.Key].OrderByDescending(x => x.Height);
              }
            }

            else if (sortingComboBox.SelectedItem.ToString().CompareTo("Clustered Height-Area") == 0) {
              // Cluster packing items with respect to height
              for (int i = 0; i < items.Count; i++) {
                double clusterRange = items[i].TargetBin.Height * delta / 100.0;
                int clusterId = Convert.ToInt32(Math.Ceiling(items[i].Height / clusterRange));

                if (clusters.ContainsKey(clusterId)) clusters[clusterId].Add(items[i]);
                else clusters.Add(clusterId, new ItemList<PackingItem> {items[i]});
              }           

              // Sort packing items inside the cluster where they belong with respect to area = width * depth 
              foreach (KeyValuePair<int, ItemList<PackingItem>> entry in clusters) {
                  clusters[entry.Key].OrderByDescending(x => x.Width * x.Depth);
              }
            }
            
            // Insert all packing items into a single list 
            var sortedItems = new ItemList<PackingItem>(clusters.OrderByDescending(x => x.Key).SelectMany(x => x.Value));
            Content.Items = sortedItems.AsReadOnly();
        }
      }
    }
  }